"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.calculateCredit = exports.calculatePricingInformation = exports.priceFormatter = void 0;
const geo_shipping_prices_1 = require("@uppfirst/geo-shipping-prices");
const decimal_js_1 = __importDefault(require("decimal.js"));
const utils_1 = require("./utils");
Object.defineProperty(exports, "priceFormatter", { enumerable: true, get: function () { return utils_1.priceFormatter; } });
const calculateShippingPrices = ({ product, stateCode, countryCode, traitCost, decimals, }) => {
    var _a, _b;
    const shipping = product.shipping;
    if (!shipping || !countryCode) {
        const zero = product.price instanceof decimal_js_1.default ? new decimal_js_1.default(0) : 0;
        return {
            taxCost: zero,
            taxPercent: zero,
            shippingCost: zero,
        };
    }
    const shippingCalculator = new geo_shipping_prices_1.Shipping(shipping);
    const prices = stateCode && countryCode === 'US'
        ? shippingCalculator.state(stateCode)
        : shippingCalculator.country(countryCode);
    const taxPercent = (0, utils_1.ensureDecimal)(((_a = prices.prices) === null || _a === void 0 ? void 0 : _a.tax) || 0);
    const shippingCost = (0, utils_1.ensureDecimal)(((_b = prices.prices) === null || _b === void 0 ? void 0 : _b.price) || 0);
    const price = (0, utils_1.ensureDecimal)(product.price);
    const calculatedShippingCost = (0, utils_1.multiplyByDecimals)(shippingCost, decimals);
    const taxableCost = price.plus(calculatedShippingCost).plus(traitCost);
    // Determine if we should return Decimal based on input type
    const shouldReturnDecimal = product.price instanceof decimal_js_1.default;
    const calculatedTaxCost = taxPercent
        .dividedBy(100)
        .times(taxableCost)
        .round();
    return {
        taxCost: shouldReturnDecimal
            ? calculatedTaxCost
            : calculatedTaxCost.toNumber(),
        taxPercent: shouldReturnDecimal ? taxPercent : taxPercent.toNumber(),
        shippingCost: calculatedShippingCost,
    };
};
/**
 * Calculate the credit gained for a product based on the quantity and the product's reward share.
 * @param product
 * @param quantity
 * @param opts
 */
const calculateCredit = (product, quantity, currency, opts) => {
    const { returnSmallestUnit } = opts || {};
    const decimals = (0, utils_1.ensureDecimal)(currency.decimals);
    const startAmountPercent = (0, utils_1.ensureDecimal)(product.startAmountPercent).dividedBy(100);
    const rewardShare = (0, utils_1.ensureDecimal)(product.rewardShare).dividedBy(100);
    const claimPercent = new decimal_js_1.default(1).minus(startAmountPercent);
    const totalBase = (0, utils_1.ensureDecimal)(product.price).times(quantity);
    const baseReserve = totalBase.times(startAmountPercent);
    const baseClaim = totalBase.times(claimPercent);
    const gainedCreditReserve = rewardShare.times(baseReserve);
    const gainedCreditClaim = rewardShare.times(baseClaim);
    return {
        gainedCreditReserve: (0, utils_1.divideConvertBackSmallestUnit)(gainedCreditReserve, decimals, returnSmallestUnit),
        gainedCreditClaim: (0, utils_1.divideConvertBackSmallestUnit)(gainedCreditClaim, decimals, returnSmallestUnit),
        gainedCredit: (0, utils_1.divideConvertBackSmallestUnit)(gainedCreditReserve.plus(gainedCreditClaim), decimals, returnSmallestUnit),
    };
};
exports.calculateCredit = calculateCredit;
const calculatePricingInformation = ({ items, currency, countryCode, stateCode, usedCreditReserve = 0, usedCreditClaim = 0, }, opts) => {
    const { returnSmallestUnit } = opts || {};
    if (items.length === 0) {
        throw new Error('No items provided for pricing calculation');
    }
    const decimals = (0, utils_1.ensureDecimal)(currency.decimals);
    // Calculate different prices for each asset item
    const itemCalculations = items.map(({ product, combination, quantity }) => {
        const startAmountPercent = (0, utils_1.ensureDecimal)(product.startAmountPercent).dividedBy(100);
        const claimPercent = new decimal_js_1.default(1).minus(startAmountPercent);
        const totalBase = (0, utils_1.ensureDecimal)(product.price).times(quantity);
        const traitCostSingle = (0, utils_1.ensureDecimal)((combination === null || combination === void 0 ? void 0 : combination.price) || 0);
        const traitCostAll = traitCostSingle.times(quantity);
        const baseReserve = totalBase.times(startAmountPercent);
        const baseClaim = totalBase.times(claimPercent);
        const { gainedCreditReserve, gainedCreditClaim, gainedCredit } = calculateCredit(product, quantity, currency, {
            returnSmallestUnit: true,
        });
        const { shippingCost, taxCost, taxPercent } = calculateShippingPrices({
            product,
            countryCode,
            stateCode,
            traitCost: traitCostSingle,
            decimals,
        });
        const shippingCostDecimal = (0, utils_1.ensureDecimal)(shippingCost).times(quantity);
        const taxCostDecimal = (0, utils_1.ensureDecimal)(taxCost).times(quantity);
        // Credit should be deducted in summary section
        const totalReserve = decimal_js_1.default.max(baseReserve
            .plus(traitCostAll)
            .plus(taxCostDecimal)
            .plus(shippingCostDecimal), new decimal_js_1.default(0));
        const total = decimal_js_1.default.max(totalReserve.plus(baseClaim), new decimal_js_1.default(0));
        return {
            prices: {
                startAmountPercent: startAmountPercent.times(100).toNumber(),
                totalBase,
                baseReserve,
                baseClaim,
                traitCost: traitCostAll,
                gainedCreditReserveDecimal: (0, utils_1.ensureDecimal)(gainedCreditReserve),
                gainedCreditClaimDecimal: (0, utils_1.ensureDecimal)(gainedCreditClaim),
                gainedCreditDecimal: (0, utils_1.ensureDecimal)(gainedCredit),
                shippingCostDecimal,
                taxCostDecimal,
                taxPercentDecimal: (0, utils_1.ensureDecimal)(taxPercent),
                totalReserve,
                total,
            },
        };
    });
    const usedCreditReserveDecimal = (0, utils_1.ensureDecimal)(usedCreditReserve);
    const usedCreditClaimDecimal = (0, utils_1.ensureDecimal)(usedCreditClaim);
    const usedCreditDecimal = usedCreditReserveDecimal.plus(usedCreditClaimDecimal);
    return {
        items: itemCalculations.map(({ prices }) => ({
            startAmountPercent: prices.startAmountPercent,
            reserve: {
                base: (0, utils_1.divideConvertBackSmallestUnit)(prices.baseReserve, decimals, returnSmallestUnit),
                traitCost: (0, utils_1.divideConvertBackSmallestUnit)(prices.traitCost, decimals, returnSmallestUnit),
                gainedCredit: (0, utils_1.divideConvertBackSmallestUnit)(prices.gainedCreditReserveDecimal, decimals, returnSmallestUnit),
                taxPercent: (0, utils_1.convertBack)(prices.taxPercentDecimal, decimals),
                shippingCost: (0, utils_1.divideConvertBackSmallestUnit)(prices.shippingCostDecimal, decimals, returnSmallestUnit),
                taxCost: (0, utils_1.divideConvertBackSmallestUnit)(prices.taxCostDecimal, decimals, returnSmallestUnit),
                // Credit should be deducted in summary section
                total: (0, utils_1.divideConvertBackSmallestUnit)(prices.totalReserve, decimals, returnSmallestUnit),
            },
            claim: {
                base: (0, utils_1.divideConvertBackSmallestUnit)(prices.baseClaim, decimals, returnSmallestUnit),
                gainedCredit: (0, utils_1.divideConvertBackSmallestUnit)(prices.gainedCreditClaimDecimal, decimals, returnSmallestUnit),
                // Credit should be deducted in summary section
                total: (0, utils_1.divideConvertBackSmallestUnit)(prices.baseClaim, decimals, returnSmallestUnit),
            },
            aggregate: {
                base: (0, utils_1.divideConvertBackSmallestUnit)(prices.totalBase, decimals, returnSmallestUnit),
                gainedCredit: (0, utils_1.divideConvertBackSmallestUnit)(prices.gainedCreditDecimal, decimals, returnSmallestUnit),
                // Credit should be deducted in summary section
                total: (0, utils_1.divideConvertBackSmallestUnit)(prices.total, decimals, returnSmallestUnit),
            },
        })),
        summary: (() => {
            const summaryTotals = itemCalculations.reduce((acc, { prices }) => ({
                // Reserve totals
                gainedCreditReserve: acc.gainedCreditReserve.plus(prices.gainedCreditReserveDecimal),
                baseReserve: acc.baseReserve.plus(prices.baseReserve),
                traitCost: acc.traitCost.plus(prices.traitCost),
                taxCost: acc.taxCost.plus(prices.taxCostDecimal),
                shippingCost: acc.shippingCost.plus(prices.shippingCostDecimal),
                totalReserve: acc.totalReserve.plus(prices.totalReserve),
                // Claim totals
                gainedCreditClaim: acc.gainedCreditClaim.plus(prices.gainedCreditClaimDecimal),
                baseClaim: acc.baseClaim.plus(prices.baseClaim),
                // Aggregate totals
                gainedCredit: acc.gainedCredit.plus(prices.gainedCreditDecimal),
                total: acc.total.plus(prices.total),
            }), {
                gainedCreditReserve: new decimal_js_1.default(0),
                baseReserve: new decimal_js_1.default(0),
                traitCost: new decimal_js_1.default(0),
                taxCost: new decimal_js_1.default(0),
                shippingCost: new decimal_js_1.default(0),
                totalReserve: new decimal_js_1.default(0),
                gainedCreditClaim: new decimal_js_1.default(0),
                baseClaim: new decimal_js_1.default(0),
                gainedCredit: new decimal_js_1.default(0),
                total: new decimal_js_1.default(0),
            });
            return {
                reserve: {
                    gainedCredit: (0, utils_1.divideConvertBackSmallestUnit)(summaryTotals.gainedCreditReserve, decimals, returnSmallestUnit),
                    base: (0, utils_1.divideConvertBackSmallestUnit)(summaryTotals.baseReserve, decimals, returnSmallestUnit),
                    traitCost: (0, utils_1.divideConvertBackSmallestUnit)(summaryTotals.traitCost, decimals, returnSmallestUnit),
                    taxCost: (0, utils_1.divideConvertBackSmallestUnit)(summaryTotals.taxCost, decimals, returnSmallestUnit),
                    shippingCost: (0, utils_1.divideConvertBackSmallestUnit)(summaryTotals.shippingCost, decimals, returnSmallestUnit),
                    total: (0, utils_1.divideConvertBackSmallestUnit)(decimal_js_1.default.max(summaryTotals.totalReserve.minus(usedCreditReserveDecimal), new decimal_js_1.default(0)), decimals, returnSmallestUnit),
                },
                claim: {
                    gainedCredit: (0, utils_1.divideConvertBackSmallestUnit)(summaryTotals.gainedCreditClaim, decimals, returnSmallestUnit),
                    base: (0, utils_1.divideConvertBackSmallestUnit)(summaryTotals.baseClaim, decimals, returnSmallestUnit),
                    total: (0, utils_1.divideConvertBackSmallestUnit)(decimal_js_1.default.max(summaryTotals.baseClaim.minus(usedCreditClaimDecimal), new decimal_js_1.default(0)), decimals, returnSmallestUnit),
                },
                aggregate: {
                    usedCredit: (0, utils_1.divideConvertBackSmallestUnit)(usedCreditDecimal, decimals, returnSmallestUnit),
                    gainedCredit: (0, utils_1.divideConvertBackSmallestUnit)(summaryTotals.gainedCredit, decimals, returnSmallestUnit),
                    base: (0, utils_1.divideConvertBackSmallestUnit)(summaryTotals.baseReserve.plus(summaryTotals.baseClaim), decimals, returnSmallestUnit),
                    total: (0, utils_1.divideConvertBackSmallestUnit)(decimal_js_1.default.max(summaryTotals.total.minus(usedCreditDecimal), new decimal_js_1.default(0)), decimals, returnSmallestUnit),
                },
            };
        })(),
    };
};
exports.calculatePricingInformation = calculatePricingInformation;
